@@ -2,7 +2,7 @@
   
    Expect_network commands
 
-   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Eloy Paris
    Copyright (C) 1987-2002 Don Libes <libes@cme.nist.gov>,
 	National Institute of Standards and Technology <http://www.nist.gov/>
 
@@ -67,76 +67,27 @@ static struct nexp_cmd_descriptor nexp_cmd_after = {
 };
 
 static void
-fill_in_framedata(frame_data *fdata, struct nexp_listener *l,
-		  const struct pcap_pkthdr *h)
+fill_in_pkthdr(struct wtap_pkthdr *phdr, const struct pcap_pkthdr *h,
+	       int ll_type)
 {
-    int ll_type;
-    struct wtap_pkthdr phdr;
-    nstime_t first_ts, prev_cap_ts; /* For "struct timeval" -> nstime_t
-				       temporary conversions */
+    memset(phdr, 0, sizeof *phdr);
 
-    phdr.len = h->len;
-    phdr.caplen = h->caplen;
-    ll_type = l->type == LISTENER_LIVE
-	      ? l->_live.datalink_type : l->_pcap.datalink_type;
-    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
-    phdr.ts.secs = h->ts.tv_sec;
-    phdr.ts.nsecs = h->ts.tv_usec*1000;
+    phdr->len = h->len;
+    phdr->caplen = h->caplen;
+    phdr->pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    assert(phdr->pkt_encap != WTAP_ENCAP_UNKNOWN);
+    phdr->ts.secs = h->ts.tv_sec;
+    phdr->ts.nsecs = h->ts.tv_usec*1000;
+    phdr->presence_flags = WTAP_HAS_TS | WTAP_HAS_CAP_LEN;
+}
 
-    frame_data_init(fdata, l->nreads /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
+static void
+fill_in_framedata(frame_data *fdata, struct nexp_listener *l,
+		  struct wtap_pkthdr *phdr)
+{
+    frame_data_init(fdata, l->nreads /* num */, phdr, 0 /* offset */, 0 /* cum_bytes */);
 
-#if 0
-    /*
-     * If we don't have the timestamp of the first packet provided by the
-     * listener it's because this is the first packet. Save the timestamp of
-     * this packet as the timestamp of the first packet.
-     */
-    if (!timerisset(&l->first_ts) )
-	l->first_ts = h->ts;
-
-    /*
-     * Listeners keep time at microseconds resolution. Wireshark stores
-     * time with nanoseconds resolution. Therefore we need to do some
-     * conversions before calculating time deltas.
-     */
-    first_ts.secs = l->first_ts.tv_sec;
-    first_ts.nsecs = l->first_ts.tv_usec*1000;
-
-    /* Get the time elapsed between the first packet and this packet. */
-    nstime_delta(&fdata->rel_ts, &fdata->abs_ts, &first_ts);
-
-    /*
-     * If we don't have the timestamp of the previous packet read from
-     * this listener it's because this is the first packet. Save the
-     * timestamp of this packet as the timestamp of the previous read
-     * packet.
-     */
-    if (!timerisset(&l->prev_cap_ts) )
-	l->prev_cap_ts = h->ts;
-
-    /*
-     * Listeners keep time at microseconds resolution. Wireshark stores
-     * time with nanoseconds resolution. Therefore we need to do some
-     * conversions before calculating time deltas.
-     */
-    prev_cap_ts.secs = l->prev_cap_ts.tv_sec;
-    prev_cap_ts.nsecs = l->prev_cap_ts.tv_usec*1000;
-
-    /*
-     * Get the time elapsed between the previous captured packet and
-     * this packet.
-     */
-    nstime_delta(&fdata->del_cap_ts, &fdata->abs_ts, &prev_cap_ts);
-
-    /*
-     * For us, delta between this packet and the previous captured packet
-     * and delta between this packet and the previous displayed packet
-     * are the same.
-     */
-    fdata->del_dis_ts = fdata->del_cap_ts;
-
-    l->prev_cap_ts = h->ts;
-#endif
+    nstime_set_unset(&fdata->rel_ts);
 }
 
 /*
@@ -148,6 +99,8 @@ static void
 process_packet(u_char *user, const struct pcap_pkthdr *h, const u_char *bytes)
 {
     struct pcap_handler_data *hd;
+    struct wtap_pkthdr phdr;
+    int ll_type;
     frame_data fdata;
     dfilter_t *rfcode;
     const char *ns_name;
@@ -157,17 +110,23 @@ process_packet(u_char *user, const struct pcap_pkthdr *h, const u_char *bytes)
     hd->listener->nreads++; /* Increment number of times we're read from
 			       this listener */
 
-    fill_in_framedata(&fdata, hd->listener, h);
+    ll_type = hd->listener->type == LISTENER_LIVE
+	      ? hd->listener->_live.datalink_type
+	      : hd->listener->_pcap.datalink_type;
+
+    fill_in_pkthdr(&phdr, h, ll_type);
+
+    fill_in_framedata(&fdata, hd->listener, &phdr);
 
     rfcode = hd->listener->type == LISTENER_LIVE ? hd->listener->_live.rfcode
 						 : hd->listener->_pcap.rfcode;
 
     ns_name = Tcl_GetVar(hd->interp, DISSECTION_NS_VARNAME, TCL_GLOBAL_ONLY);
 
-    hd->passed_rfilter = pkt_dissect_tcl(hd->interp, bytes, &fdata, rfcode,
-					 ns_name, 1);
+    hd->passed_rfilter = pkt_dissect_tcl(hd->interp, bytes, &phdr, &fdata,
+					 rfcode, ns_name, 1);
 
-    frame_data_cleanup(&fdata);
+    frame_data_destroy(&fdata);
 
     /*
      * We rely on the current time in some operations (when reading packets
@@ -244,34 +203,34 @@ dump_nexp_cmd_descriptor(struct nexp_cmd_descriptor *ecd)
 
 	ec = ecd->cases[i];
 
-	printf("    Case #%d: (at %p)\n", i, ecd->cases[i]);
+	printf("    Case #%d: (at %p)\n", i, ec);
 
-	printf("      Listener: '%s'\n", ecd->cases[i]->listener->name);
+	printf("      Listener: '%s'\n", ec->listener->name);
 
 	if (ecd->cases[i]->pat)
 	    printf("      Expression: '%s' (at %p)\n",
-		   Tcl_GetString(ecd->cases[i]->pat), ecd->cases[i]->pat);
+		   Tcl_GetString(ec->pat), ec->pat);
 	else
 	    printf("      Expression: none\n");
 
 	if (ecd->cases[i]->body)
 	    printf("      Body: %s (at %p)\n",
-		   Tcl_GetString(ecd->cases[i]->body), ecd->cases[i]->body);
+		   Tcl_GetString(ec->body), ec->body);
 	else
 	    printf("      Body: none\n");
 
-	if (ecd->cases[i]->use > (sizeof(uses)/sizeof(uses[0]) ) )
+	if (ec->use > (sizeof(uses)/sizeof(uses[0]) ) )
 	    use = "Invalid";
 	else
-	    use = uses[ecd->cases[i]->use];
-	printf("      use: %s (%d)\n", use, ecd->cases[i]->use);
+	    use = uses[ec->use];
+	printf("      use: %s (%d)\n", use, ec->use);
 
 	printf("      Write indices: %s\n",
-	       ecd->cases[i]->indices ? "yes" : "no");
+	       ec->indices ? "yes" : "no");
 	printf("      Re-read indirects: %s\n",
-	       ecd->cases[i]->iread ? "yes" : "no");
+	       ec->iread ? "yes" : "no");
 	printf("      Write timestamps: %s\n",
-	       ecd->cases[i]->timestamp ? "yes" : "no");
+	       ec->timestamp ? "yes" : "no");
     }
 
     printf("\n");
@@ -1658,7 +1617,7 @@ static int
 NExp_ExpectNetworkBackgroundObjCmd(ClientData clientData, Tcl_Interp *interp,
 				   int objc, Tcl_Obj *CONST objv[])
 {
-    int pid, retval;
+    int pid;
     struct nexp_cmd_descriptor eg;
     Tcl_Obj *obj;
 
@@ -1747,7 +1706,7 @@ NExp_ExpectNetworkBackgroundObjCmd(ClientData clientData, Tcl_Interp *interp,
 
     setpgid(0 /* use the child's PID */, pgid);
 
-    retval = expect_network(interp, &eg);
+    expect_network(interp, &eg);
 
     free_ecases(&eg);
 
