@@ -3,7 +3,7 @@
 
 /* tcl_packet.h -- Definitions for the "packet" Tcl command
 
-   Copyright (C) 2007, 2008, 2009, 2010 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Eloy Paris
 
    This is part of Network Expect (nexp)
 
@@ -25,7 +25,8 @@
 #include <pcap.h>
 
 extern Tcl_Obj *Tcl_NewPacketObj(const uint8_t *, struct pcap_pkthdr *, int);
-extern int Tcl_GetPacketFromObj(Tcl_Obj *, frame_data *, uint8_t **);
+extern int Tcl_GetPacketFromObj(Tcl_Obj *, struct wtap_pkthdr *,
+				frame_data *, uint8_t **);
 
 extern void nexp_init_packet_cmd(Tcl_Interp *interp);
 
