@@ -3,7 +3,7 @@
    Interface with libwireshark for packet dissection. Packet dissection
    is launched from here but is ultimately performed by libwireshark.
 
-   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Eloy Paris
 
    Some code here has been borrowed from the Wireshark source code, and
    is copyright Gerald Combs and others.
@@ -27,6 +27,8 @@
 
 #include "includes.h"
 
+#include <wiretap/pcap-encap.h> /* For wtap_wtap_encap_to_pcap_encap() */
+
 #include "tcl_ws-ftypes.h"
 #include "packets-priv.h"
 
@@ -389,11 +391,11 @@ proto_tree_tcl(Tcl_Interp *interp, epan_dissect_t *edt, const char *ns_name,
  *	!= 0 otherwise.
  */
 int
-pkt_dissect_tcl(Tcl_Interp *interp, const u_char *packet, frame_data *fdata,
+pkt_dissect_tcl(Tcl_Interp *interp, const u_char *packet,
+		struct wtap_pkthdr *wphdr, frame_data *fdata,
 		dfilter_t *rfcode, const char *ns_name, int create_non_ws_vars)
 {
     epan_dissect_t *edt;
-    union wtap_pseudo_header pseudo_header;
     Tcl_Obj *obj;
     int passed;
     struct pcap_pkthdr pkt_hdr;
@@ -406,8 +408,6 @@ pkt_dissect_tcl(Tcl_Interp *interp, const u_char *packet, frame_data *fdata,
     if (ns_name && !Tcl_FindNamespace(interp, ns_name, NULL, TCL_GLOBAL_ONLY) )
 	Tcl_CreateNamespace(interp, ns_name, NULL, NULL);
 
-    memset(&pseudo_header, 0, sizeof(pseudo_header) );
-
     edt = epan_dissect_new(1 /* create_proto_tree */,
 			   1 /* proto_tree_visible */);
 
@@ -415,7 +415,7 @@ pkt_dissect_tcl(Tcl_Interp *interp, const u_char *packet, frame_data *fdata,
 	epan_dissect_prime_dfilter(edt, rfcode);
 
     /* Outsource the heavy lifting to libwireshark */
-    epan_dissect_run(edt, &pseudo_header, packet, fdata, NULL /* cinfo */);
+    epan_dissect_run(edt, wphdr, packet, fdata, NULL /* cinfo */);
 
     /* Run the read filter if we have one. */
     passed = rfcode ? dfilter_apply_edt(rfcode, edt) : 1;
@@ -448,7 +448,8 @@ pkt_dissect_tcl(Tcl_Interp *interp, const u_char *packet, frame_data *fdata,
 	pkt_hdr.ts.tv_sec = fdata->abs_ts.secs;
 	pkt_hdr.ts.tv_usec = fdata->abs_ts.nsecs/1000;
 
-	obj = Tcl_NewPacketObj(packet, &pkt_hdr, fdata->lnk_t);
+	obj = Tcl_NewPacketObj(packet, &pkt_hdr,
+			       wtap_wtap_encap_to_pcap_encap(fdata->lnk_t) );
 	varname = _pkt_prepend_ns_name(ns_name, PACKETRAW_VARNAME);
 	Tcl_SetVar2Ex(interp, varname, NULL, obj, 0);
 	free(varname);
@@ -468,6 +469,7 @@ _pkt_get_uint8(Tcl_Interp *interp, const char *varname, uint8_t *p)
 
     obj = Tcl_GetVar2Ex(interp, varname, NULL, 0);
     if (obj) {
+	/* FIXME: check return from Tcl_GetUInt8FromObj() */
 	Tcl_GetUInt8FromObj(obj, p);
 	return 0;
     } else
@@ -481,6 +483,7 @@ _pkt_get_uint16(Tcl_Interp *interp, const char *varname, uint16_t *p)
 
     obj = Tcl_GetVar2Ex(interp, varname, NULL, 0);
     if (obj) {
+	/* FIXME: check return from Tcl_GetUInt16FromObj() */
 	Tcl_GetUInt16FromObj(obj, p);
 	*p = htons(*p);
 	return 0;
@@ -488,6 +491,27 @@ _pkt_get_uint16(Tcl_Interp *interp, const char *varname, uint16_t *p)
 	return -1;
 }
 
+#if 0
+int
+_pkt_get_list_uint16(Tcl_Interp *interp, const char *varname, int index,
+		     uint16_t *p)
+{
+    Tcl_Obj *obj, *list_obj;
+
+    list_obj = Tcl_GetVar2Ex(interp, varname, NULL, 0);
+    if (list_obj) {
+	if (Tcl_ListObjIndex(interp, list_obj, index, &obj) == TCL_ERROR
+	    || obj == NULL)
+	    return -1;
+
+	if (Tcl_GetUInt16FromObj(obj, p) == TCL_ERROR) return -1;
+	*p = htons(*p);
+	return 0;
+    } else
+	return -1;
+}
+#endif
+
 int
 _pkt_get_ipaddr(Tcl_Interp *interp, const char *varname, ip_addr_t *p)
 {
@@ -495,6 +519,7 @@ _pkt_get_ipaddr(Tcl_Interp *interp, const char *varname, ip_addr_t *p)
 
     obj = Tcl_GetVar2Ex(interp, varname, NULL, 0);
     if (obj) {
+	/* FIXME: check return from Tcl_GetIPAddrFromObj() */
 	Tcl_GetIPAddrFromObj(obj, p);
 	*p = htonl(*p);
 	return 0;
@@ -509,6 +534,7 @@ _pkt_get_boolean(Tcl_Interp *interp, const char *varname, int *p)
 
     obj = Tcl_GetVar2Ex(interp, varname, NULL, 0);
     if (obj) {
+	/* FIXME: check return from Tcl_GetBooleanFromObj() */
 	Tcl_GetBooleanFromObj(interp, obj, p);
 	return 0;
     } else
@@ -637,19 +663,17 @@ proto_tree_print_simple_node(proto_node *node, gpointer data)
  * fdata: frame_data structure (initialized by caller)
  */
 void
-pkt_dissect_show(const u_char *packet, frame_data *fdata, int verbose)
+pkt_dissect_show(const u_char *packet, struct wtap_pkthdr *wphdr,
+		 frame_data *fdata, int verbose)
 {
     epan_dissect_t *edt;
-    union wtap_pseudo_header pseudo_header;
     struct callback_data_show cb_data;
 
-    memset(&pseudo_header, 0, sizeof(pseudo_header) );
-
     edt = epan_dissect_new(1 /* create_proto_tree */,
 			   1 /* proto_tree_visible */);
 
     /* Outsource the heavy lifting to libwireshark */
-    epan_dissect_run(edt, &pseudo_header, packet, fdata, NULL /* cinfo */);
+    epan_dissect_run(edt, wphdr, packet, fdata, NULL /* cinfo */);
 
     /*
      * The packet has been dissected. Now print results.
