@@ -2,7 +2,8 @@
  
    PDU builder for ICMPv6 messages (RFC 2461 and 2462)
 
-   Copyright (C) 2007, 2008, 2009 Eloy Paris, Darrin Miller
+   Copyright (C) 2007, 2008 Darrin Miller
+   Copyright (C) 2007, 2008, 2009, 2010 Eloy Paris
 
    This is part of Network Expect.
 
@@ -41,19 +42,24 @@ MTU                                        5
 /*
  * Most ICMPv6 builders only need to calculate the ICMPv6 checksum in the
  * second pass. This generic second pass builder can be used for all these
- * ICMPv6 builders.
+ * ICMPv6 builders. Note that if there are extension headers, the IPv6
+ * header needed to be able to calculate the ICMPv6 checksum is not
+ * immediately above the ICMPv6 PDU so a little extra work needs to
+ * be done to find it.
  */
 static void
 pdu_icmpv6_generic_pass2_builder(const GNode *pdu,
 				 void *dest,
-				 void *prev_pdu_hdr)
+				 void *prev_pdu_hdr _U_)
 {
     struct ipv6_pseudo_hdr *ph;
     struct ip6_hdr *ip6;
     struct icmpv6_hdr *icmpv6;
     size_t hdr_len, opts_len, payload_len, total_length;
     uint16_t cksum;
-    void *field;
+    void *field, *parent_pdu_dest;
+    struct node_data *node_data;
+    GNode *parent_pdu;
 
     icmpv6 = dest;
 
@@ -66,37 +72,57 @@ pdu_icmpv6_generic_pass2_builder(const GNode *pdu,
     else {
 	/*
 	 * If we don't have a numspec for the checksum then we calculate
-	 * the correct checksum.
+	 * the correct checksum. The challenge here is that we need to
+	 * access the IPv6 header. For this we go up the PDU tree.
 	 */
 
-	ip6 = prev_pdu_hdr;
+	for (ip6 = NULL, parent_pdu_dest = dest, parent_pdu = pdu->parent;
+	     parent_pdu;
+	     parent_pdu = parent_pdu->parent) {
+	    node_data = parent_pdu->data;
 
-	hdr_len = ( (struct node_data *) pdu->data)->_data_pdu.hdr_len;
-	opts_len = ( (struct node_data *) pdu->data)->_data_pdu.opts_len;
-	payload_len = ( (struct node_data *) pdu->data)->_data_pdu.payload_len;
+	    parent_pdu_dest = (void *) parent_pdu_dest
+			      - node_data->_data_pdu.opts_len
+			      - node_data->_data_pdu.hdr_len;
 
-	total_length = hdr_len + payload_len + opts_len; 
+	    if (!strcmp(node_data->name, "ip6") ) {
+		ip6 = parent_pdu_dest;
+		break;
+	    }
+	}
 
-	ph = xmalloc(sizeof(struct ipv6_pseudo_hdr) + total_length);
+	if (ip6) {
+	    hdr_len = ( (struct node_data *) pdu->data)->_data_pdu.hdr_len;
+	    opts_len = ( (struct node_data *) pdu->data)->_data_pdu.opts_len;
+	    payload_len = ( (struct node_data *) pdu->data)->_data_pdu.payload_len;
 
-	ph->saddr = ip6->ip6_src;
-	ph->daddr = ip6->ip6_dst;
-	ph->len = htonl(total_length);
-	ph->zero1 = 0;
-	ph->zero2 = 0;
-	ph->nxt = IPPROTO_ICMPV6;
+	    total_length = hdr_len + payload_len + opts_len; 
 
-	/* Put the IPv6 Pseudo Header and the ICMPv6 PDU (dest) together */
-	memcpy( (unsigned char *) ph + sizeof(struct ipv6_pseudo_hdr),
-	       dest, total_length);
+	    ph = xmalloc(sizeof(struct ipv6_pseudo_hdr) + total_length);
 
-	/* Must be 0 before calculating cksum! */
-	SSVAL(icmpv6, offsetof(struct icmpv6_hdr, sum), 0);
+	    ph->saddr = ip6->ip6_src;
+	    ph->daddr = ip6->ip6_dst;
+	    ph->len = htonl(total_length);
+	    ph->zero1 = 0;
+	    ph->zero2 = 0;
+	    ph->nxt = IPPROTO_ICMPV6;
 
-	/* Do a checksum on the new pseudo_hdr + ICMPv6 message */
-	cksum = _pb_cksum(ph, sizeof(struct ipv6_pseudo_hdr) + total_length);
+	    /* Put the IPv6 Pseudo Header and the ICMPv6 PDU (dest) together */
+	    memcpy( (unsigned char *) ph + sizeof(struct ipv6_pseudo_hdr),
+		   dest, total_length);
 
-	free(ph);
+	    /* Must be 0 before calculating cksum! */
+	    SSVAL(icmpv6, offsetof(struct icmpv6_hdr, sum), 0);
+
+	    /* Do a checksum on the new pseudo_hdr + ICMPv6 message */
+	    cksum = _pb_cksum(ph, sizeof(struct ipv6_pseudo_hdr) + total_length);
+
+	    free(ph);
+	} else
+	    /*
+	     * Could not find the IPv6 header; put an arbitrary checksum.
+	     */
+	    cksum = -1;
     }
 
     SSVAL(icmpv6, offsetof(struct icmpv6_hdr, sum), cksum);
@@ -335,6 +361,7 @@ static void
 pdu_icmpv6redirect_builder(const GNode *pdu, void *dest)
 {
     struct icmpv6redirect_hdr *icmpv6;
+    ip6_addr_t *ip6addr;
 
     icmpv6 = dest;
 
@@ -344,8 +371,11 @@ pdu_icmpv6redirect_builder(const GNode *pdu, void *dest)
     /* Per RFC 2461 */
     SIVAL(icmpv6, offsetof(struct icmpv6redirect_hdr, reserved), 0);
 
-    icmpv6->target = *(ip6_addr_t *) _pb_pdata(pdu, "target");
-    icmpv6->dest = *(ip6_addr_t *) _pb_pdata(pdu, "dst");
+    if ( (ip6addr = _pb_pdata(pdu, "target") ) )
+	icmpv6->target = *ip6addr;
+
+    if ( (ip6addr = _pb_pdata(pdu, "dst") ) )
+	icmpv6->dest = *ip6addr;
 }
 
 #if 0
@@ -377,13 +407,15 @@ static void
 pdu_nd_neighbor_solicit_builder(const GNode *pdu, void *dest)
 {
     struct nd_neighbor_solicit_hdr *icmpv6;
+    ip6_addr_t *ip6addr;
 
     icmpv6 = dest;
 
     icmpv6->type = ND_NEIGHBOR_SOLICIT;
     icmpv6->code = 0;
     icmpv6->reserved = 0; /* Per RFC 2461 */
-    icmpv6->target = *(ip6_addr_t *) _pb_pdata(pdu, "target");
+    if ( (ip6addr = _pb_pdata(pdu, "target") ) )
+	icmpv6->target = *ip6addr;
 }
 
 #if 0
@@ -412,6 +444,7 @@ static void
 pdu_nd_neighbor_advert_builder(const GNode *pdu, void *dest)
 {
     struct nd_neighbor_advert_hdr *icmpv6;
+    ip6_addr_t *ip6addr;
 
     icmpv6 = dest;
 
@@ -422,7 +455,8 @@ pdu_nd_neighbor_advert_builder(const GNode *pdu, void *dest)
     icmpv6->o = (unsigned) _pb_pdata(pdu, "o");
     icmpv6->reserved1 = 0; /* Per RFC 2461 */
     icmpv6->reserved2 = 0; /* Per RFC 2461 */
-    icmpv6->target = *(ip6_addr_t *) _pb_pdata(pdu, "target");
+    if ( (ip6addr = _pb_pdata(pdu, "target") ) )
+	icmpv6->target = *ip6addr;
 }
 
 #if 0
@@ -486,6 +520,8 @@ static void
 pdu_nd_router_advert_builder(const GNode *pdu, void *dest)
 {
     struct nd_router_advert_hdr *icmpv6;
+    uint16_t u16;
+    uint32_t u32;
 
     icmpv6 = dest;
 
@@ -498,14 +534,14 @@ pdu_nd_router_advert_builder(const GNode *pdu, void *dest)
     icmpv6->prf = num_next(_pb_pdata(pdu, "prf") );
     icmpv6->reserved = 0; /* Per RFC 2461, 3775, router selection draft */
 
-    SSVAL(icmpv6, offsetof(struct nd_router_advert_hdr, router_lifetime),
-	  num_next(_pb_pdata(pdu, "lifetime") ) );
+    u16 = htons(num_next(_pb_pdata(pdu, "lifetime") ) );
+    SSVAL(icmpv6, offsetof(struct nd_router_advert_hdr, router_lifetime), u16);
 
-    SIVAL(icmpv6, offsetof(struct nd_router_advert_hdr, reachable_time), 
-	  num_next(_pb_pdata(pdu, "reachable-time") ) );
+    u32 = htonl(num_next(_pb_pdata(pdu, "reachable-time") ) );
+    SIVAL(icmpv6, offsetof(struct nd_router_advert_hdr, reachable_time), u32);
 
-    SIVAL(icmpv6, offsetof(struct nd_router_advert_hdr, retrans_timer),
-	  num_next(_pb_pdata(pdu, "retrans-time") ) );
+    u32 = htonl(num_next(_pb_pdata(pdu, "retrans-time") ) );
+    SIVAL(icmpv6, offsetof(struct nd_router_advert_hdr, retrans_timer), u32);
 }
 
 #if 0
@@ -616,12 +652,12 @@ icmpv6_tlv(const GNode *option, void *dest,
 
     tlvptr = dest;
 
-    /* TVL type */
+    /* TLV type */
     *tlvptr++ = num_next(_pb_pdata(option, "type") );
 
     data = _pb_pdata(option, "value");
     if (data) {
-	/* TVL length */
+	/* TLV length */
 	if ( (field = _pb_pdata(option, "length") ) )
 	    /* User specified a fake length */
 	    *tlvptr++ = num_next(field);
@@ -635,7 +671,7 @@ icmpv6_tlv(const GNode *option, void *dest,
 
 	tlvptr += data->len;
     } else {
-	/* TVL length */
+	/* TLV length */
 	if ( (field = _pb_pdata(option, "length") ) )
 	    /* User specified a fake length */
 	    *tlvptr++ = num_next(field);
@@ -644,6 +680,43 @@ icmpv6_tlv(const GNode *option, void *dest,
     }
 }
 
+static void
+icmpv6_mtu(const GNode *option, void *dest,
+           void *curr_pdu_hdr _U_, void *prev_pdu _U_)
+{
+    uint8_t *tlvptr;
+    uint32_t u32;
+
+    tlvptr = dest;
+
+    *tlvptr++ = 5; /* TLV type */
+    *tlvptr++ = 8 /* type (1) + length (1) + padding (2) + MTU (4) */ / 8;
+    *tlvptr++ = '\0'; /* padding */
+    *tlvptr++ = '\0'; /* padding */
+
+    u32 = htonl(num_next( ( (struct node_data *) option->data)->_data_parm.data) );
+    SIVAL(tlvptr, 0, u32);
+}
+
+static void
+icmpv6_lnklayeraddr(const GNode *option, void *dest,
+		    void *curr_pdu_hdr _U_, void *prev_pdu _U_)
+{
+    uint8_t *tlvptr;
+
+    tlvptr = dest;
+
+    *tlvptr++ = 1; /* TLV type */
+    *tlvptr++ = 8 /* type (1) + length (1) + MAC addr (6) */ / 8;
+    memcpy(tlvptr, ( (struct node_data *) option->data)->_data_parm.data,
+	   ETH_ADDR_LEN);
+}
+
+
+/*********************************************************************
+ *** PDU definitions (basic ICMPv6)
+ ********************************************************************/
+
 static pdu_t pdu_icmpv6raw = {
     .name = "icmp6",
     .description = "Raw ICMPv6 message",
@@ -770,6 +843,30 @@ static pdu_t pdu_icmpv6redirect = {
     .postbuild = &pdu_icmpv6_generic_pass2_builder,
 };
 
+/*********************************************************************
+ *** PDU definitions (Neighbor Discovery)
+ ********************************************************************/
+
+/* First some options used by all the ND PDUs */
+
+static field_t icmp6_options[] = {
+    {.name = "mtu", .type = PDU_FTYPE_NUMTYPE},
+    {.name = "src-link-layer-addr", .type = PDU_FTYPE_MACADDR},
+    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
+	{.name = "type", .type = PDU_FTYPE_NUMTYPE},
+	{.name = "length", .type = PDU_FTYPE_NUMTYPE},
+	{.name = "value", .type = PDU_FTYPE_DATA},
+	{.name = NULL} }, .defval = NULL},
+    {.name = NULL}
+};
+
+/*
+ * Then all the ND PDUs. Note how the .data field of the "options"
+ * field references the above options. This is done to avoid creating
+ * the same options over and over each time a PDU is defined, i.e.
+ * change options in a single place and all PDUs benefit.
+ */
+
 static pdu_t pdu_nd_neighbor_solicit = {
     .name = "icmp6-ns",
     .description = "ICMPv6 Neighbor Solicitation",
@@ -778,13 +875,8 @@ static pdu_t pdu_nd_neighbor_solicit = {
     .fields = (field_t []) {
 	{.name = "cksum", .type = PDU_FTYPE_NUMTYPE},
 	{.name = "target", .type = PDU_FTYPE_IP6ADDR},
-	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-	    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-		{.name = "type", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "length", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "value", .type = PDU_FTYPE_DATA},
-		{.name = NULL} }, .defval = NULL},
-	    {.name = NULL} }, .defval = NULL},
+	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS,
+	 .data = &icmp6_options},
 	{.name = NULL}
     },
     .options_class = "icmp6",
@@ -803,13 +895,8 @@ static pdu_t pdu_nd_neighbor_advert = {
 	{.name = "s", .type = PDU_FTYPE_BIT},
 	{.name = "o", .type = PDU_FTYPE_BIT},
 	{.name = "target", .type = PDU_FTYPE_IP6ADDR},
-	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-	    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-		{.name = "type", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "length", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "value", .type = PDU_FTYPE_DATA},
-		{.name = NULL} }, .defval = NULL},
-	    {.name = NULL} }, .defval = NULL},
+	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS,
+	 .data = &icmp6_options},
 	{.name = NULL}
     },
     .options_class = "icmp6",
@@ -824,13 +911,8 @@ static pdu_t pdu_nd_router_solicit = {
     .len = sizeof(struct nd_router_solicit_hdr),
     .fields = (field_t []) {
 	{.name = "cksum", .type = PDU_FTYPE_NUMTYPE},
-	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-	    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-		{.name = "type", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "length", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "value", .type = PDU_FTYPE_DATA},
-		{.name = NULL} }, .defval = NULL},
-	    {.name = NULL} }, .defval = NULL},
+	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS,
+	 .data = &icmp6_options},
 	{.name = NULL}
     },
     .options_class = "icmp6",
@@ -853,13 +935,8 @@ static pdu_t pdu_nd_router_advert = {
 	{.name = "lifetime", .type = PDU_FTYPE_NUMTYPE},
 	{.name = "reachable-time", .type = PDU_FTYPE_NUMTYPE},
 	{.name = "retrans-time", .type = PDU_FTYPE_NUMTYPE},
-	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-	    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-		{.name = "type", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "length", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "value", .type = PDU_FTYPE_DATA},
-		{.name = NULL} }, .defval = NULL},
-	    {.name = NULL} }, .defval = NULL},
+	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS,
+	 .data = &icmp6_options},
 	{.name = NULL}
     },
     .options_class = "icmp6",
@@ -883,13 +960,8 @@ static pdu_t pdu_icmpv6_router_renumber = {
 	{.name = "s", .type = PDU_FTYPE_BIT},
 	{.name = "p", .type = PDU_FTYPE_BIT},
 	{.name = "max-delay", .type = PDU_FTYPE_NUMTYPE},
-	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-	    {.name = "tlv", .type = PDU_FTYPE_BRACED_ARGS, .data = (field_t []) {
-		{.name = "type", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "length", .type = PDU_FTYPE_NUMTYPE},
-		{.name = "value", .type = PDU_FTYPE_DATA},
-		{.data = NULL} }, .defval = NULL},
-	    {.name = NULL} }, .defval = NULL},
+	{.name = "options", .type = PDU_FTYPE_BRACED_ARGS,
+	 .data = &icmp6_options},
 	{.name = NULL}
     },
     .options_class = "icmp6",
@@ -897,13 +969,31 @@ static pdu_t pdu_icmpv6_router_renumber = {
     .postbuild = &pdu_icmpv6_generic_pass2_builder,
 };
 
+/*********************************************************************
+ *** PDU options
+ ********************************************************************/
+
 static const pdu_option_t pdu_option_icmpv6tlv = {
     .name = "tlv",
-    .description = "ICMPv6 type-length-value (TLV) options",
+    .description = "ICMPv6 type-length-value (TLV) option",
     .build = &icmpv6_tlv,
     .get_len = &pdu_generic_tlv_len
 };
 
+static const pdu_option_t pdu_option_icmpv6mtu = {
+    .name = "mtu",
+    .description = "ICMPv6 MTU option",
+    .build = &icmpv6_mtu,
+    .len = 8 /* type (1) + length (1) + padding (2) + MTU (4) */
+};
+
+static const pdu_option_t pdu_option_icmpv6_lnklayeraddr = {
+    .name = "src-link-layer-addr",
+    .description = "ICMPv6 Source Link Layer Address option",
+    .build = &icmpv6_lnklayeraddr,
+    .len = 8 /* type (1) + length (1) + MAC addr (6) */
+};
+
 void
 _pb_register_icmp6(void)
 {
@@ -922,4 +1012,6 @@ _pb_register_icmp6(void)
     _pb_register_protocol(&pdu_icmpv6_router_renumber);
 
     _pb_register_protocol_option("icmp6", &pdu_option_icmpv6tlv);
+    _pb_register_protocol_option("icmp6", &pdu_option_icmpv6mtu);
+    _pb_register_protocol_option("icmp6", &pdu_option_icmpv6_lnklayeraddr);
 }
