@@ -2,7 +2,7 @@
  
    PDU builder for IPv6 packets
 
-   Copyright (C) 2007, 2008, 2009 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011 Eloy Paris
 
    This is part of Network Expect.
 
@@ -62,7 +62,7 @@ ipv6_nxt_hdr(const GNode *pdu)
     void *field;
     const GNode *next_pdu;
 
-    if (  !(field = _pb_pdata(pdu, "proto") )
+    if (  !(field = _pb_pdata(pdu, "next-header") )
 	&& (next_pdu = pb_nextpdu(pdu) ) ) {
 	/*
 	 * User didn't specify the IP protocol so we try to put in the right
@@ -163,7 +163,7 @@ pdu_ipv6hbh_builder(const GNode *pdu, void *dest)
 {
     struct ip6_hbh *hbh;
     uint8_t *optsptr;
-    const GNode *child, *tlv;
+    const GNode *child;
     struct node_data *node_data;
     struct payload *data;
     size_t hdr_len;
@@ -171,7 +171,7 @@ pdu_ipv6hbh_builder(const GNode *pdu, void *dest)
 
     hbh = (struct ip6_hbh *) dest;
 
-    hbh->ip6h_nxt = num_next(_pb_pdata(pdu, "next-header") );
+    hbh->ip6h_nxt = ipv6_nxt_hdr(pdu);
 
     optsptr = (uint8_t *) hbh + sizeof(struct ip6_hbh);
 
@@ -184,7 +184,12 @@ pdu_ipv6hbh_builder(const GNode *pdu, void *dest)
 	    *optsptr++ = IP6OPT_PAD1;
 	} else if (!strcmp(node_data->name, "padn") ) {
 	    *optsptr++ = IP6OPT_PADN;
-	    *optsptr++ = num_next(node_data->_data_parm.data);
+	    /*
+	     * Substract 2 per RFC 2460 section 4.2, which says "For N octets
+	     * of padding, the Opt Data Len field contains the value N-2, and
+	     * the Option Data consists of N-2 zero-valued octets.
+	     */
+	    *optsptr++ = num_next(node_data->_data_parm.data) - 2;
 	    memset(optsptr, 0, optsptr[-1]);
 	    optsptr += optsptr[-1];
 	} else if (!strcmp(node_data->name, "raw-tlv") ) {
@@ -193,34 +198,30 @@ pdu_ipv6hbh_builder(const GNode *pdu, void *dest)
 		   ( (struct payload *) node_data->_data_parm.data)->len);
 	    optsptr += ( (struct payload *) node_data->_data_parm.data)->len;
 	} else if (!strcmp(node_data->name, "tlv") ) {
-	    for (tlv = g_node_first_child(child);
-		 tlv;
-		 tlv = g_node_next_sibling(child) ) {
-		/* TVL type */
-		*optsptr++ = num_next(_pb_pdata(tlv, "type") );
+	    /* TVL type */
+	    *optsptr++ = num_next(_pb_pdata(child, "type") );
 
-		data = _pb_pdata(tlv, "value");
-		if (data) {
-		    /* TVL length */
-		    if ( (field = _pb_pdata(tlv, "length") ) )
-			/* User specified a fake length */
-			*optsptr++ = num_next(field);
-		    else
-			/* Use real length */
-			*optsptr++ = data->len;
+	    data = _pb_pdata(child, "value");
+	    if (data) {
+		/* TVL length */
+		if ( (field = _pb_pdata(child, "length") ) )
+		    /* User specified a fake length */
+		    *optsptr++ = num_next(field);
+		else
+		    /* Use real length */
+		    *optsptr++ = data->len;
 
-		    /* TLV data */
-		    memcpy(optsptr, data->data, data->len);
+		/* TLV data */
+		memcpy(optsptr, data->data, data->len);
 
-		    optsptr += data->len;
-		} else {
-		    /* TVL length */
-		    if ( (field = _pb_pdata(tlv, "length") ) )
-			/* User specified a fake length */
-			*optsptr++ = num_next(field);
-		    else
-			*optsptr++ = 0;
-		}
+		optsptr += data->len;
+	    } else {
+		/* TVL length */
+		if ( (field = _pb_pdata(child, "length") ) )
+		    /* User specified a fake length */
+		    *optsptr++ = num_next(field);
+		else
+		    *optsptr++ = 0;
 	    }
 	}
     }
@@ -237,7 +238,7 @@ pdu_ipv6hbh_builder(const GNode *pdu, void *dest)
 static size_t
 pdu_ipv6hbh_len(const GNode *pdu)
 {
-    const GNode *child, *tlv;
+    const GNode *child;
     struct node_data *node_data;
     struct payload *data;
     size_t hdr_len;
@@ -253,20 +254,16 @@ pdu_ipv6hbh_len(const GNode *pdu)
 	    hdr_len++;
 	} else if (!strcmp(node_data->name, "padn") ) {
 	    hdr_len += 1 /* type */ + 1 /* length */
-			+ num_next(node_data->_data_parm.data) /* padding */;
+		       + num_next(node_data->_data_parm.data) - 2 /* padding */;
 	} else if (!strcmp(node_data->name, "raw-tlv") ) {
 	    hdr_len += ( (struct payload *) node_data->_data_parm.data)->len;
 	} else if (!strcmp(node_data->name, "tlv") ) {
-	    for (tlv = g_node_first_child(child);
-		 tlv;
-		 tlv = g_node_next_sibling(child) ) {
-		data = _pb_pdata(child, "value");
+	    data = _pb_pdata(child, "value");
 
-		hdr_len += 1 /* type */ + 1 /* length */;
+	    hdr_len += 1 /* type */ + 1 /* length */;
 
-		if (data)
-		    hdr_len += data->len;
-	    }
+	    if (data)
+		hdr_len += data->len;
 	}
     }
 
@@ -282,7 +279,7 @@ pdu_ipv6destopts_builder(const GNode *pdu, void *dest)
 {
     struct ip6_dest *destopts;
     uint8_t *optsptr;
-    const GNode *child, *tlv;
+    const GNode *child;
     struct node_data *node_data;
     struct payload *data;
     size_t hdr_len;
@@ -290,7 +287,7 @@ pdu_ipv6destopts_builder(const GNode *pdu, void *dest)
 
     destopts = (struct ip6_dest *) dest;
 
-    destopts->ip6d_nxt = num_next(_pb_pdata(pdu, "next-header") );
+    destopts->ip6d_nxt = ipv6_nxt_hdr(pdu);
 
     optsptr = (uint8_t *) destopts + sizeof(struct ip6_dest);
 
@@ -303,7 +300,12 @@ pdu_ipv6destopts_builder(const GNode *pdu, void *dest)
 	    *optsptr++ = IP6OPT_PAD1;
 	} else if (!strcmp(node_data->name, "padn") ) {
 	    *optsptr++ = IP6OPT_PADN;
-	    *optsptr++ = num_next(node_data->_data_parm.data);
+	    /*
+	     * Substract 2 per RFC 2460 section 4.2, which says "For N octets
+	     * of padding, the Opt Data Len field contains the value N-2, and
+	     * the Option Data consists of N-2 zero-valued octets.
+	     */
+	    *optsptr++ = num_next(node_data->_data_parm.data) - 2;
 	    memset(optsptr, 0, optsptr[-1]);
 	    optsptr += optsptr[-1];
 	} else if (!strcmp(node_data->name, "raw-tlv") ) {
@@ -312,35 +314,30 @@ pdu_ipv6destopts_builder(const GNode *pdu, void *dest)
 		   ( (struct payload *) node_data->_data_parm.data)->len);
 	    optsptr += ( (struct payload *) node_data->_data_parm.data)->len;
 	} else if (!strcmp(node_data->name, "tlv") ) {
-	    for (tlv = g_node_first_child(child);
-		 tlv;
-		 tlv = g_node_next_sibling(child) ) {
+	    /* TVL type */
+	    *optsptr++ = num_next(_pb_pdata(child, "type") );
 
-		/* TVL type */
-		*optsptr++ = num_next(_pb_pdata(child, "type") );
+	    data = _pb_pdata(child, "value");
+	    if (data) {
+		/* TVL length */
+		if ( (field = _pb_pdata(child, "length") ) )
+		    /* User specified a fake length */
+		    *optsptr++ = num_next(field);
+		else
+		    /* Use real length */
+		    *optsptr++ = data->len;
 
-		data = _pb_pdata(child, "value");
-		if (data) {
-		    /* TVL length */
-		    if ( (field = _pb_pdata(child, "length") ) )
-			/* User specified a fake length */
-			*optsptr++ = num_next(field);
-		    else
-			/* Use real length */
-			*optsptr++ = data->len;
+		/* TLV data */
+		memcpy(optsptr, data->data, data->len);
 
-		    /* TLV data */
-		    memcpy(optsptr, data->data, data->len);
-
-		    optsptr += data->len;
-		} else {
-		    /* TVL length */
-		    if ( (field = _pb_pdata(child, "length") ) )
-			/* User specified a fake length */
-			*optsptr++ = num_next(field);
-		    else
-			*optsptr++ = 0;
-		}
+		optsptr += data->len;
+	    } else {
+		/* TVL length */
+		if ( (field = _pb_pdata(child, "length") ) )
+		    /* User specified a fake length */
+		    *optsptr++ = num_next(field);
+		else
+		    *optsptr++ = 0;
 	    }
 	}
     }
@@ -363,7 +360,7 @@ pdu_ipv6frag_builder(const GNode *pdu, void *dest)
 
     frag = (struct ip6_frag *) dest;
 
-    frag->ip6f_nxt = num_next(_pb_pdata(pdu, "next-header") );
+    frag->ip6f_nxt = ipv6_nxt_hdr(pdu);
 
     u32 = htonl(num_next(_pb_pdata(pdu, "id") ) );
     SIVAL(frag, offsetof(struct ip6_frag, ip6f_ident), u32);
@@ -371,7 +368,7 @@ pdu_ipv6frag_builder(const GNode *pdu, void *dest)
     u16 = 0;
     if (_pb_pdata(pdu, "more") )
 	u16 |= IP6F_MORE_FRAG;
-    u16 |= htons(num_next(_pb_pdata(pdu, "fragoff") ) << 3);
+    u16 |= htons(num_next(_pb_pdata(pdu, "offset") ) << 3);
     SSVAL(frag, offsetof(struct ip6_frag, ip6f_offlg), u16);
 }
 
@@ -408,7 +405,7 @@ pdu_ipv6rthdr_builder(const GNode *pdu, void *dest)
 
     rthdr = (struct ip6_rthdr *) dest;
 
-    rthdr->ip6r_nxt = num_next(_pb_pdata(pdu, "next-header") );
+    rthdr->ip6r_nxt = ipv6_nxt_hdr(pdu);
     rthdr->ip6r_type = num_next(_pb_pdata(pdu, "type") );
     rthdr->ip6r_segleft = num_next(_pb_pdata(pdu, "segs-left") );
 
