@@ -9,7 +9,7 @@
    stores sent and received packets in raw (binary) form, but the basics
    are the same.
 
-   Copyright (C) 2007, 2008, 2009, 2010 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Eloy Paris
 
    This is part of Network Expect.
 
@@ -260,21 +260,27 @@ error:
 }
 
 static void
-fill_in_framedata(frame_data *fdata, const struct pcap_pkthdr *h, int ll_type)
+fill_in_pkthdr(struct wtap_pkthdr *phdr, const struct pcap_pkthdr *h,
+	       int ll_type)
 {
-    struct wtap_pkthdr phdr;
+    memset(phdr, 0, sizeof *phdr);
 
-    phdr.len = h->len;
-    phdr.caplen = h->caplen;
-    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
-    phdr.ts.secs = h->ts.tv_sec;
-    phdr.ts.nsecs = h->ts.tv_usec*1000;
+    phdr->len = h->len;
+    phdr->caplen = h->caplen;
+    phdr->pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    assert(phdr->pkt_encap != WTAP_ENCAP_UNKNOWN);
+    phdr->ts.secs = h->ts.tv_sec;
+    phdr->ts.nsecs = h->ts.tv_usec*1000;
+    phdr->presence_flags = WTAP_HAS_TS | WTAP_HAS_CAP_LEN;
+}
 
-    frame_data_init(fdata, 0 /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
+static void
+//fill_in_framedata(frame_data *fdata, const struct pcap_pkthdr *h, int ll_type)
+fill_in_framedata(frame_data *fdata, struct wtap_pkthdr *phdr)
+{
+    frame_data_init(fdata, 0 /* num */, phdr, 0 /* offset */, 0 /* cum_bytes */);
 
     nstime_set_unset(&fdata->rel_ts);
-    nstime_set_unset(&fdata->del_cap_ts);
-    nstime_set_unset(&fdata->del_dis_ts);
 }
 
 /*
@@ -289,6 +295,7 @@ build_and_hash(Tcl_Interp *interp, struct sent_packet **sent_packets)
     int npackets;
     frame_data fdata;
     struct pcap_pkthdr pkthdr;
+    struct wtap_pkthdr phdr;
 
     /*
      * Calculate total number of packets that we will send.
@@ -313,11 +320,14 @@ build_and_hash(Tcl_Interp *interp, struct sent_packet **sent_packets)
 	pkthdr.len = pkthdr.caplen;
 	gettimeofday(&pkthdr.ts, NULL);
 
-	fill_in_framedata(&fdata, &pkthdr, packets[n].ll_type);
+	fill_in_pkthdr(&phdr, &pkthdr, packets[n].ll_type);
 
-	packets[n].hash = pkt_hash(interp, packets[n].packet.data, &fdata);
+	fill_in_framedata(&fdata, &phdr);
 
-	frame_data_cleanup(&fdata);
+	packets[n].hash = pkt_hash(interp, packets[n].packet.data, &phdr,
+				   &fdata);
+
+	frame_data_destroy(&fdata);
 
 #ifdef DEBUG
 	printf("sent pkt len = %u, packet data:\n", packets[n].packet.len);
@@ -426,10 +436,13 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
     struct timeval time_remaining, *tremainingptr;
     int got_answer;
     int nanswers;
+#ifdef SELECT_LIES
     int call_select, got_packet;
+#endif
     int fd;
     int datalink_type;
     pcap_t *pd;
+    struct wtap_pkthdr rcvd_wphdr, sent_wphdr;
     frame_data rcvd_fdata, sent_fdata;
     size_t hsize_adj; /* Layer 2 header size adjustment */
     Tcl_Obj *sent_list, *received_list, *unanswered_list, *packet_obj;
@@ -540,12 +553,15 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 
 	thread_created++;
 
+#ifdef SELECT_LIES
+	 call_select = 0; /* Work around FreeBSD silliness */
+#endif
+
 	/*
 	 * Receive loop. We loop while we still have unanswered packets
 	 * and while we have not timed out.
 	 */
-	for (timerclear(&timeout_time),
-	     call_select = 0 /* Work around FreeBSD silliness */;
+	for (timerclear(&timeout_time);
 
 	     /*
 	      * This condition looks a bit complicated, but it is not.
@@ -681,12 +697,14 @@ bypass_select:
 		    /* FIXME: need to check for error here? */
 		    continue;
 
+#ifdef SELECT_LIES
 		/*
 		 * This will be used to determine whether we need to call
 		 * select() next time (workaround for what seems to be a
 		 * FreeBSD silliness.)
 		 */
 		got_packet = 1;
+#endif
 
 		/*
 		 * Determine the layer 2 and layer 3 PDUs as well as the size
@@ -700,11 +718,11 @@ bypass_select:
 		 */
 		datalink_type = llist[i]->_live.datalink_type;
 
-		fill_in_framedata(&rcvd_fdata, &pkthdr, datalink_type);
+		fill_in_pkthdr(&rcvd_wphdr, &pkthdr, datalink_type);
 
-		hash = pkt_hash(interp, pkt_data, &rcvd_fdata);
+		fill_in_framedata(&rcvd_fdata, &rcvd_wphdr);
 
-		frame_data_cleanup(&rcvd_fdata);
+		hash = pkt_hash(interp, pkt_data, &rcvd_wphdr, &rcvd_fdata);
 
 #ifdef DEBUG
 		printf("recv'ed pkt len = %u, packet data:\n", pkthdr.len);
@@ -788,14 +806,16 @@ bypass_select:
 		    sent_pkthdr.len = sent_pkthdr.caplen;
 		    gettimeofday(&sent_pkthdr.ts, NULL);
 
-		    fill_in_framedata(&sent_fdata, &sent_pkthdr,
-				      sent_packets[j].ll_type);
+		    fill_in_pkthdr(&sent_wphdr, &sent_pkthdr,
+				   sent_packets[j].ll_type);
 
-		    isanswer = pkt_isanswer(interp, pkt_data, &rcvd_fdata,
-					    sent_packets[j].packet.data,
-					    &sent_fdata);
+		    fill_in_framedata(&sent_fdata, &sent_wphdr);
+
+		    isanswer = pkt_isanswer(interp,
+					    pkt_data, &rcvd_wphdr, &rcvd_fdata,
+					    sent_packets[j].packet.data, &sent_wphdr, &sent_fdata);
 		    
-		    frame_data_cleanup(&sent_fdata);
+		    frame_data_destroy(&sent_fdata);
 
 		    if (!isanswer)
 			/*
@@ -840,17 +860,21 @@ bypass_select:
 
 		g_byte_array_free(hash, TRUE);
 
+		frame_data_destroy(&rcvd_fdata);
+
 		if (!got_answer)
 		    if (vflag)
 			putchar('.');
 	    } /* Listener sweep loop */
 
+#ifdef SELECT_LIES
 	    /*
 	     * If we read at least one packet then we need to bypass select()
 	     * again. We only call select() if we tried to read from all file
 	     * descriptors and no data was available anywhere.
 	     */
 	    call_select = !got_packet;
+#endif
 	} /* Receive loop */
 
 	if (vflag)
@@ -1021,6 +1045,7 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
     int nanswers;
     int datalink_type;
     pcap_t *pd;
+    struct wtap_pkthdr rcvd_wphdr, sent_wphdr;
     frame_data rcvd_fdata, sent_fdata;
     size_t hsize_adj; /* Layer 2 header size adjustment */
     Tcl_Obj *sent_list, *received_list, *unanswered_list, *packet_obj;
@@ -1222,11 +1247,11 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 		 */
 		datalink_type = llist[i]->_live.datalink_type;
 
-		fill_in_framedata(&rcvd_fdata, &pkthdr, datalink_type);
+		fill_in_pkthdr(&rcvd_wphdr, &pkthdr, datalink_type);
 
-		hash = pkt_hash(interp, pkt_data, &rcvd_fdata);
+		fill_in_framedata(&rcvd_fdata, &rcvd_wphdr);
 
-		frame_data_cleanup(&rcvd_fdata);
+		hash = pkt_hash(interp, pkt_data, &rcvd_wphdr, &rcvd_fdata);
 
 #ifdef DEBUG
 		printf("recv'ed pkt len = %u, packet data:\n", pkthdr.len);
@@ -1310,14 +1335,16 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 		    sent_pkthdr.len = sent_pkthdr.caplen;
 		    gettimeofday(&sent_pkthdr.ts, NULL);
 
-		    fill_in_framedata(&sent_fdata, &sent_pkthdr,
-				      sent_packets[j].ll_type);
+		    fill_in_pkthdr(&sent_wphdr, &sent_pkthdr,
+				   sent_packets[j].ll_type);
 
-		    isanswer = pkt_isanswer(interp, pkt_data, &rcvd_fdata,
-					    sent_packets[j].packet.data,
-					    &sent_fdata);
+		    fill_in_framedata(&sent_fdata, &sent_wphdr);
 
-		    frame_data_cleanup(&sent_fdata);
+		    isanswer = pkt_isanswer(interp,
+					    pkt_data, &rcvd_wphdr, &rcvd_fdata,
+					    sent_packets[j].packet.data, &sent_wphdr, &sent_fdata);
+
+		    frame_data_destroy(&sent_fdata);
 
 		    if (!isanswer)
 			/*
@@ -1360,6 +1387,8 @@ NExp_SendExpectObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int argc,
 		    got_answer = 1;
 		} /* Hash search loop */
 
+		frame_data_destroy(&rcvd_fdata);
+
 		g_byte_array_free(hash, TRUE);
 
 		if (!got_answer)
