@@ -184,22 +184,10 @@ dumpx(const struct payload *p, const struct formatspec *f, unsigned *offset)
 		}
 		break;
 	    case FORMAT_DECIMAL:
-		if (f->size == sizeof(uint8_t) ) {
-		    format = "0x%d ";
-		} else if (f->size == sizeof(uint16_t) ) {
-		    format = "0x%d ";
-		} else {
-		    format = "0x%d ";
-		}
+		format = "0x%d ";
 		break;
 	    case FORMAT_UNSIGNED:
-		if (f->size == sizeof(uint8_t) ) {
-		    format = "0x%u ";
-		} else if (f->size == sizeof(uint16_t) ) {
-		    format = "0x%u ";
-		} else {
-		    format = "0x%u ";
-		}
+		format = "0x%u ";
 		break;
 	    default:
 		/* Just to shut the compiler up. We should never end up here */
@@ -211,11 +199,11 @@ dumpx(const struct payload *p, const struct formatspec *f, unsigned *offset)
 	    } else if (f->size == sizeof(uint8_t) ) {
 		value = ( (uint8_t *) ptr)[i*objects_per_line + j];
 	    } else if (f->size == sizeof(uint16_t) ) {
-		value = SVAL(ptr, (i*objects_per_line + j)*f->size);
+		value = SVAL(ptr, (i*objects_per_line + j)*sizeof(uint16_t) );
 		if (!f->littleendian)
 		    value = htons(value);
 	    } else {
-		value = IVAL(ptr, (i*objects_per_line + j)*f->size);
+		value = IVAL(ptr, (i*objects_per_line + j)*sizeof(uint32_t) );
 		if (!f->littleendian)
 		    value = htonl(value);
 	    }
@@ -574,13 +562,17 @@ tcl_barray_examine(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     if (obj->typePtr != &tclBArrayType) {
 	nexp_error(interp, "\"%s\" is not a barray object",
-		   objv[1]);
+		   Tcl_GetString(objv[1]) );
 	return TCL_ERROR;
     }
 
     p = obj->internalRep.otherValuePtr;
 
     if (objc == 3) {
+	/*
+	 * Argument #3 could either be a format (argument starts with '/'),
+	 * or an offset (argument does not start with '/').
+	 */
 	if (Tcl_GetString(objv[2])[0] == '/') {
 	    if (parse_format(Tcl_GetString(objv[2]), &f) == -1) {
 		nexp_error(interp, "invalid /FMT");
@@ -590,6 +582,11 @@ tcl_barray_examine(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 	    offset = strtoul(Tcl_GetString(objv[2]), NULL, 0);
 	}
     } if (objc == 4) {
+	/*
+	 * Both a format and an offset have been provided. This means that
+	 * argument #3 has to be a format, i.e. first character must be
+	 * '/'.
+	 */
 	if (Tcl_GetString(objv[2])[0] != '/')
 	    goto usage;
 
@@ -989,9 +986,125 @@ usage:
     return TCL_ERROR;
 }
 
+/********************************************************************
+ *				 peek                               *
+ ********************************************************************/
+
+/*
+ * Very similar to the "barray examine" command, but instead of
+ * dumping packet contents to stdout it leaves packet contents
+ * in Tcl variables.
+ */
+static int
+NExp_PeekObjCmd(ClientData clientData _U_, Tcl_Interp *interp, int objc,
+		Tcl_Obj * const *objv)
+{
+    Tcl_Obj *obj, *return_obj;
+    static unsigned offset = 0;
+    static struct formatspec f = {
+	.count = 1,
+	.format = FORMAT_HEX,
+	.size = 1,
+	.littleendian = 0
+    };
+    struct payload *p;
+    void *ptr;
+    uint32_t value;
+    unsigned i;
+
+    if (objc < 2 || objc > 4)
+	goto usage;
+
+    obj = Tcl_ObjGetVar2(interp, objv[1], NULL, TCL_LEAVE_ERR_MSG);
+    if (!obj)
+	return TCL_ERROR;
+
+    if (obj->typePtr != &tclBArrayType) {
+	nexp_error(interp, "\"%s\" is not a barray object",
+		   Tcl_GetString(objv[1]) );
+	return TCL_ERROR;
+    }
+
+    p = obj->internalRep.otherValuePtr;
+
+    if (objc == 3) {
+	/*
+	 * Argument #3 could either be a format (argument starts with '/'),
+	 * or an offset (argument does not start with '/').
+	 */
+	if (Tcl_GetString(objv[2])[0] == '/') {
+	    if (parse_format(Tcl_GetString(objv[2]), &f) == -1) {
+		nexp_error(interp, "invalid /FMT");
+		return TCL_ERROR;
+	    }
+	} else {
+	    offset = strtoul(Tcl_GetString(objv[2]), NULL, 0);
+	}
+    } if (objc == 4) {
+	/*
+	 * Both a format and an offset have been provided. This means that
+	 * argument #3 has to be a format, i.e. first character must be
+	 * '/'.
+	 */
+	if (Tcl_GetString(objv[2])[0] != '/')
+	    goto usage;
+
+	if (parse_format(Tcl_GetString(objv[2]), &f) == -1) {
+	    nexp_error(interp, "invalid /FMT");
+	    return TCL_ERROR;
+	}
+
+	offset = strtoul(Tcl_GetString(objv[3]), NULL, 0);
+    }
+
+    if (offset + f.count*f.size > p->len) {
+	nexp_error(interp, "offset/count/size combination would access data "
+			   "outside packet boundary");
+	return TCL_ERROR;
+    }
+
+    for (ptr = p->data + offset, return_obj = NULL, i = 0; i < f.count; i++) {
+	if (f.format == FORMAT_STRING) {
+	    obj = Tcl_NewStringObj(ptr, strlen(ptr) );
+	} else if (f.size == sizeof(uint8_t) ) {
+	    value = ( (uint8_t *) ptr)[i];
+	    obj = Tcl_NewIntObj(value);
+	} else if (f.size == sizeof(uint16_t) ) {
+	    value = SVAL(ptr, i*sizeof(uint16_t) );
+	    if (!f.littleendian)
+		value = htons(value);
+	    obj = Tcl_NewIntObj(value);
+	} else {
+	    value = IVAL(ptr, i*sizeof(uint32_t) );
+	    if (!f.littleendian)
+		value = htonl(value);
+	    obj = Tcl_NewIntObj(value);
+	}
+
+	if (f.count == 1)
+	    return_obj = obj;
+	else {
+	    if (!return_obj)
+		return_obj = Tcl_NewListObj(0, NULL);
+
+	    Tcl_ListObjAppendElement(interp, return_obj, obj);
+	}
+    }
+
+    Tcl_SetObjResult(interp, return_obj);
+
+    return TCL_OK;
+
+usage:
+    nexp_error(interp, "usage: peek <barray variable> [/FMT] [offset]");
+    return TCL_ERROR;
+}
+
+
 static struct nexp_cmd_data cmd_data[] = {
     {"barray", NExp_BArrayObjCmd, NULL, 0, 0},
     {"x", NULL, NExp_XCmd, 0, 0},
+    {"peek", NExp_PeekObjCmd, NULL, 0, 0},
 #if 0
     {"e", NULL, NExp_ECmd, 0, 0},
 #endif
