@@ -5,7 +5,7 @@
    The "packet" and "ws" Tcl commands and the "packet" Tcl object type
    are implemented here.
 
-   Copyright (C) 2007, 2008, 2009, 2010 Eloy Paris
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013 Eloy Paris
 
    Some code here has been borrowed from the Wireshark source code, and
    is copyright Gerald Combs and others.
@@ -29,6 +29,7 @@
 
 #include "includes.h"
 
+#include <assert.h>
 #include <epan/prefs.h>
 #include <epan/column-utils.h>
 #include <wiretap/pcap-encap.h> /* For wtap_pcap_encap_to_wtap_encap() */
@@ -121,23 +122,29 @@ get_line_buf(size_t len)
 }
 
 static void
-fill_in_framedata(frame_data *fdata, const struct pcap_pkthdr *h, int ll_type)
+fill_in_pkthdr(struct wtap_pkthdr *phdr, const struct pcap_pkthdr *h,
+	       int ll_type)
 {
-    struct wtap_pkthdr phdr;
+    memset(phdr, 0, sizeof *phdr);
 
-    phdr.len = h->len;
-    phdr.caplen = h->caplen;
-    phdr.pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
-    phdr.ts.secs = h->ts.tv_sec;
-    phdr.ts.nsecs = h->ts.tv_usec*1000;
-
-    frame_data_init(fdata, 0 /* num */, &phdr, 0 /* offset */, 0 /* cum_bytes */);
-
-    nstime_set_unset(&fdata->rel_ts);
-    nstime_set_unset(&fdata->del_cap_ts);
-    nstime_set_unset(&fdata->del_dis_ts);
+    phdr->len = h->len;
+    phdr->caplen = h->caplen;
+    phdr->pkt_encap = wtap_pcap_encap_to_wtap_encap(ll_type);
+    assert(phdr->pkt_encap != WTAP_ENCAP_UNKNOWN);
+    phdr->ts.secs = h->ts.tv_sec;
+    phdr->ts.nsecs = h->ts.tv_usec*1000;
+    phdr->presence_flags = WTAP_HAS_TS | WTAP_HAS_CAP_LEN;
 }
 
+static void
+fill_in_framedata(frame_data *fdata, struct wtap_pkthdr *phdr)
+{
+    frame_data_init(fdata, 0 /* num */, phdr, 0 /* offset */, 0 /* cum_bytes */);
+
+    nstime_set_unset(&fdata->rel_ts);
+}
+
+/* From tshark.c:print_columns() */
 static
 char *columns_to_string(column_info *cinfo)
 {
@@ -351,20 +358,20 @@ update_string(Tcl_Obj *packet_obj)
     epan_dissect_t *edt;
     char *string_rep;
     frame_data fdata;
-    union wtap_pseudo_header pseudo_header;
+    struct wtap_pkthdr phdr;
 
     packet = packet_obj->internalRep.otherValuePtr;
 
-    memset(&pseudo_header, 0, sizeof(pseudo_header) );
+    fill_in_pkthdr(&phdr, &packet->pkthdr, packet->dl_type);
 
-    fill_in_framedata(&fdata, &packet->pkthdr, packet->dl_type);
+    fill_in_framedata(&fdata, &phdr);
 
     edt = epan_dissect_new(0 /* create_proto_tree */,
                            0 /* proto_tree_visible */);
 
     col_custom_prime_edt(edt, &cinfo);
 
-    epan_dissect_run(edt, &pseudo_header, packet->bytes.data, &fdata, &cinfo);
+    epan_dissect_run(edt, &phdr, packet->bytes.data, &fdata, &cinfo);
 
     epan_dissect_fill_in_columns(edt, FALSE, TRUE);
 
@@ -381,7 +388,7 @@ update_string(Tcl_Obj *packet_obj)
     packet_obj->length = len;
 
     epan_dissect_free(edt);
-    frame_data_cleanup(&fdata);
+    frame_data_destroy(&fdata);
 }
 
 static void
@@ -424,7 +431,8 @@ Tcl_NewPacketObj(const uint8_t *data, struct pcap_pkthdr *pkthdr, int dl_type)
 }
 
 int
-Tcl_GetPacketFromObj(Tcl_Obj *obj, frame_data *fdata, uint8_t **pkt)
+Tcl_GetPacketFromObj(Tcl_Obj *obj, struct wtap_pkthdr *phdr,
+		     frame_data *fdata, uint8_t **pkt)
 {
     struct packet *packet;
 
@@ -433,7 +441,10 @@ Tcl_GetPacketFromObj(Tcl_Obj *obj, frame_data *fdata, uint8_t **pkt)
 
     packet = obj->internalRep.otherValuePtr;
 
-    fill_in_framedata(fdata, &packet->pkthdr, packet->dl_type);
+    fill_in_pkthdr(phdr, &packet->pkthdr, packet->dl_type);
+
+    fill_in_framedata(fdata, phdr);
+
     *pkt = packet->bytes.data;
 
     return 0;
@@ -457,6 +468,7 @@ tcl_packet_dissect(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
     enum options {
 	OPT_NAMESPACE
     };
+    struct wtap_pkthdr phdr;
 
     /*
      * Parse command-line arguments.
@@ -494,7 +506,9 @@ tcl_packet_dissect(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     packet = obj->internalRep.otherValuePtr;
 
-    fill_in_framedata(&fdata, &packet->pkthdr, packet->dl_type);
+    fill_in_pkthdr(&phdr, &packet->pkthdr, packet->dl_type);
+
+    fill_in_framedata(&fdata, &phdr);
 
     if (!ns_name)
 	/*
@@ -503,9 +517,10 @@ tcl_packet_dissect(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 	 */
 	ns_name = Tcl_GetVar(interp, DISSECTION_NS_VARNAME, TCL_GLOBAL_ONLY);
 
-    pkt_dissect_tcl(interp, packet->bytes.data, &fdata, NULL, ns_name, 1);
+    pkt_dissect_tcl(interp, packet->bytes.data, &phdr, &fdata, NULL,
+		    ns_name, 1);
 
-    frame_data_cleanup(&fdata);
+    frame_data_destroy(&fdata);
 
     return TCL_OK;
 }
@@ -517,6 +532,7 @@ tcl_packet_hash(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
     Tcl_Obj *packet_obj, *barray_obj;
     GByteArray *hash;
     frame_data fdata;
+    struct wtap_pkthdr phdr;
 
     if (objc != 2) {
 	nexp_error(interp, "usage: packet hash <packet variable>");
@@ -535,9 +551,11 @@ tcl_packet_hash(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     packet = packet_obj->internalRep.otherValuePtr;
 
-    fill_in_framedata(&fdata, &packet->pkthdr, packet->dl_type);
+    fill_in_pkthdr(&phdr, &packet->pkthdr, packet->dl_type);
 
-    hash = pkt_hash(interp, packet->bytes.data, &fdata);
+    fill_in_framedata(&fdata, &phdr);
+
+    hash = pkt_hash(interp, packet->bytes.data, &phdr, &fdata);
 
     barray_obj = Tcl_NewBArrayObj(hash->data, hash->len);
 
@@ -743,6 +761,7 @@ tcl_packet_show(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
     struct packet *packet;
     Tcl_Obj *obj;
     frame_data fdata;
+    struct wtap_pkthdr phdr;
     int i, index, verbose;
     static const char *options[] = {
 	"-verbose", NULL
@@ -782,11 +801,13 @@ tcl_packet_show(Tcl_Interp *interp, int objc, Tcl_Obj * const *objv)
 
     packet = obj->internalRep.otherValuePtr;
 
-    fill_in_framedata(&fdata, &packet->pkthdr, packet->dl_type);
+    fill_in_pkthdr(&phdr, &packet->pkthdr, packet->dl_type);
 
-    pkt_dissect_show(packet->bytes.data, &fdata, verbose);
+    fill_in_framedata(&fdata, &phdr);
 
-    frame_data_cleanup(&fdata);
+    pkt_dissect_show(packet->bytes.data, &phdr, &fdata, verbose);
+
+    frame_data_destroy(&fdata);
 
     return TCL_OK;
 }
